﻿using Microsoft.AspNetCore.Components;
using Microsoft.AspNetCore.WebUtilities;
using Microsoft.JSInterop;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace Blazor.Pages
{
    public class InternalNavigationManager
    {
        public NavigationManager NavigationManager { get; }

        private readonly IJSRuntime jsRuntime;

        private readonly QueryParamState queryParamState;


        public InternalNavigationManager(
            NavigationManager navigationManager,
            IJSRuntime jsRuntime,
            QueryParamState queryParamState)
        {
            NavigationManager = navigationManager;
            this.jsRuntime = jsRuntime;
            this.queryParamState = queryParamState;
        }

        public bool TryGetQueryParam(string key, out string value)
        {
            TryGetQueryParam(
                NavigationManager.Uri,
                key,
                out string queryParamValue);

            value = queryParamValue;

            return !string.IsNullOrEmpty(queryParamValue);
        }

        public void NavigateTo(string url)
        {
            NavigationManager.NavigateTo(url);
        }

        public void UpdateParamInUri(string queryParam, string paramValue, bool navigateToUri = true)
        {
            string baseUri = UriWithoutParameters(NavigationManager.Uri);

            queryParamState.SetState(queryParam, paramValue, baseUri);

            if (!navigateToUri) return;
            UpdateUri();
        }

        public void RemoveParamFromQuery(string queryParamName)
        {
            QueryParamData queryParamData = queryParamState.RemoveParam(queryParamName);
            if (queryParamData == null)
                return;

            string baseUri = UriWithoutParameters(NavigationManager.Uri);
            if (baseUri != queryParamData.ParentUri)
                return;

            UpdateUri();
        }

        private void UpdateUri()
        {
            Uri uri = new Uri(NavigationManager.Uri);
            List<KeyValuePair<string, string>> queryItems = ParseQueryParams(uri);

            RemoveStaleQueryParams(queryItems);
            UpdateQueryParamValue(queryItems);

            string updatedUrl = MergeUrlWithParams(uri, queryItems);
            NavigateInternallyTo(updatedUrl);
        }

        private void RemoveStaleQueryParams(List<KeyValuePair<string, string>> queryItems)
        {
            queryItems.RemoveAll(qi => !queryParamState.QueryParams.ContainsKey(qi.Key));
        }

        private void UpdateQueryParamValue(List<KeyValuePair<string, string>> queryItems)
        {
            foreach (KeyValuePair<string, QueryParamData> paramStateEntry in queryParamState.QueryParams)
            {
                queryItems.RemoveAll(item => item.Key == paramStateEntry.Key);
                queryItems.Add(new KeyValuePair<string, string>(paramStateEntry.Key, paramStateEntry.Value.Value));
            }
        }

        public virtual void NavigateInternallyTo(string url)
        {
            //for more info https://chrissainty.com/an-in-depth-look-at-routing-in-blazor/
            bool forceLoad = false;// If true, bypasses client-side routing and forces the browser to load the new
                                   // page from the server, whether or not the URI would normally be handled by the
                                   // client-side router.
            //jsRuntime.InvokeVoidAsync(InteropMethods.BlazorNavigationJsFunction, url, forceLoad);
        }

        private static string MergeUrlWithParams(Uri uri, List<KeyValuePair<string, string>> queryItems)
        {
            Dictionary<string, string> dictionary = queryItems.ToDictionary(kv => kv.Key, kv => kv.Value);
            string baseUri = uri.GetComponents(UriComponents.Path, UriFormat.UriEscaped);

            string updatedUrl = QueryHelpers.AddQueryString(baseUri, dictionary);

            return updatedUrl;
        }

        private static List<KeyValuePair<string, string>> ParseQueryParams(Uri uri)
        {
            Dictionary<string, Microsoft.Extensions.Primitives.StringValues> queryItemsDictionary = QueryHelpers.ParseQuery(uri.Query);
            List<KeyValuePair<string, string>> queryItems = queryItemsDictionary
                .SelectMany(item => item.Value, (col, value) => new KeyValuePair<string, string>(col.Key, value)).ToList();

            return queryItems;
        }

        private string UriWithoutParameters(string uriString)
        {
            Uri uri = new Uri(uriString);
            string baseUri = uri.GetComponents(UriComponents.Path, UriFormat.UriEscaped);
            return baseUri;
        }

        public static bool TryGetQueryParam<T>(string url, string key, out T value)
        {
            Uri uri = new Uri(url);
            if (QueryHelpers.ParseQuery(uri.Query).TryGetValue(key, out Microsoft.Extensions.Primitives.StringValues valueFromQueryString))
            {
                if ((typeof(T) == typeof(int) || typeof(T) == typeof(int?)) && int.TryParse(valueFromQueryString, out int valueAsInt))
                {
                    value = (T)(object)valueAsInt;
                    return true;
                }

                if (typeof(T) == typeof(string))
                {
                    value = (T)(object)valueFromQueryString.ToString();
                    return true;
                }

                if (typeof(T) == typeof(decimal) && decimal.TryParse(valueFromQueryString, out decimal valueAsDecimal))
                {
                    value = (T)(object)valueAsDecimal;
                    return true;
                }

                if (typeof(T) == typeof(bool) && bool.TryParse(valueFromQueryString, out bool valueAsBool))
                {
                    value = (T)(object)valueAsBool;
                    return true;
                }
            }

            value = default;
            return false;
        }
    }
}
