﻿using Microsoft.AspNetCore.Components;
using Microsoft.AspNetCore.WebUtilities;
using Syncfusion.Blazor.Navigations;
using System;
using System.Linq;

namespace Blazor.Pages
{
    public class RoutedTab : Tab, IDisposable
    {
        [Parameter]
        public string QueryParamName { get; set; }

        [Inject]
        public InternalNavigationManager InternalNavigationManager { get; set; }

        public override void HandleTabSelectionChanged(SelectEventArgs eventArgs)
        {
            base.HandleTabSelectionChanged(eventArgs);
            UpdateNavigationHistory();
        }

        private void UpdateNavigationHistory()
        {
            string paramValue = SelectedTabIndex < 0 || SelectedTabIndex > TabChildren.Count
                ? TabChildren[0].QueryParamIdentifier
                : TabChildren[SelectedTabIndex].QueryParamIdentifier;

            InternalNavigationManager.UpdateParamInUri(QueryParamName, paramValue);
        }

        protected override void OnAfterRender(bool firstRender)
        {
            if (!firstRender)
            {
                return;
            }

            CheckAllRequiredParametersAreSet();

            NavigateToChildTab();
        }

        private void CheckAllRequiredParametersAreSet()
        {
            if (TabChildren.Any(child => string.IsNullOrEmpty(child.QueryParamIdentifier)))
                throw new Exception("RoutedTab_TabChid_MissingParameterExceptionMessage");
            if (string.IsNullOrWhiteSpace(QueryParamName))
                throw new Exception("RoutedTab_MissingParameterExceptionMessage");
        }

        private async void NavigateToChildTab()
        {
            TryGetQueryParam(
                InternalNavigationManager.NavigationManager.Uri,
                QueryParamName,
                out string selectedTab);

            if (string.IsNullOrEmpty(selectedTab))
            {
                return;
            }

            int index = TabChildren.FindIndex(child => child.QueryParamIdentifier == selectedTab);

            if (index < 0)
            {
                InternalNavigationManager.RemoveParamFromQuery(QueryParamName);
                return;
            }

            await SwitchTo(index);
            InternalNavigationManager.UpdateParamInUri(QueryParamName, selectedTab, false);
        }

        public void Dispose()
        {
            InternalNavigationManager.RemoveParamFromQuery(QueryParamName);
        }

        public static bool TryGetQueryParam<T>(string url, string key, out T value)
        {
            Uri uri = new Uri(url);
            if (QueryHelpers.ParseQuery(uri.Query).TryGetValue(key, out Microsoft.Extensions.Primitives.StringValues valueFromQueryString))
            {
                if ((typeof(T) == typeof(int) || typeof(T) == typeof(int?)) && int.TryParse(valueFromQueryString, out int valueAsInt))
                {
                    value = (T)(object)valueAsInt;
                    return true;
                }

                if (typeof(T) == typeof(string))
                {
                    value = (T)(object)valueFromQueryString.ToString();
                    return true;
                }

                if (typeof(T) == typeof(decimal) && decimal.TryParse(valueFromQueryString, out decimal valueAsDecimal))
                {
                    value = (T)(object)valueAsDecimal;
                    return true;
                }

                if (typeof(T) == typeof(bool) && bool.TryParse(valueFromQueryString, out bool valueAsBool))
                {
                    value = (T)(object)valueAsBool;
                    return true;
                }
            }

            value = default;
            return false;
        }
    }
}
